% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modify.R
\name{modify.default}
\alias{modify.default}
\title{Modify Attributes of Indicated Components by Default}
\usage{
\method{modify}{default}(
  x,
  ...,
  .reserved = getOption("yamlet_modify_reserved", c("class", "levels", "labels",
    "names"))
)
}
\arguments{
\item{x}{object}

\item{...}{indicated columns, or name-value pairs}

\item{.reserved}{reserved labels that warn on assignment}
}
\value{
same class as x
}
\description{
Modifies the attributes of each indicated element
(all elements by default).  Tries to assign the value of an expression
to the supplied label, with existing attributes
and the object itself (.) available as arguments.
Gives a warning if the supplied label is considered reserved.
Intends to support anything with one or more non-empty names.
}
\details{
The name of the component itself is available during assignments as
attribute 'name' (any pre-existing attribute 'name' is temporarily masked).
After all assignments are complete, the value of 'name' is enforced at the object level.
Thus, \code{modify} expressions can modify component names.
}
\examples{
library(magrittr)
library(dplyr)
file <- system.file(package = 'yamlet', 'extdata','quinidine.csv')
x <- decorate(file)

# modify selected columns
x \%<>\% modify(title = paste(label, '(', guide, ')'), time)
x \%>\% select(time, conc) \%>\% decorations

# modify (almost) all columns
x \%<>\% modify(title = paste(label, '(', guide, ')'), -Subject)
x \%>\% select(time, conc) \%>\% decorations

# use column itself
x \%<>\% modify(`defined values` = sum(!is.na(.)))
x \%>\% select(time) \%>\% decorations

# rename column
x \%<>\% modify(time, name = label)
names(x)

# warn if assignment fails
\dontrun{
\donttest{
x \%<>\% modify(title = foo, time)
}}
# support lists
list(a = 1, b = 1:10, c = letters) \%>\%
modify(length = length(.), b:c)

}
\seealso{
Other modify: 
\code{\link{modify}()},
\code{\link{named}()},
\code{\link{selected.default}()},
\code{\link{selected}()}

Other interface: 
\code{\link{decorate.character}()},
\code{\link{decorate.data.frame}()},
\code{\link{ggplot.decorated}()},
\code{\link{io_csv.character}()},
\code{\link{io_csv.data.frame}()},
\code{\link{io_res.character}()},
\code{\link{io_table.character}()},
\code{\link{io_table.data.frame}()},
\code{\link{io_yamlet.character}()},
\code{\link{io_yamlet.data.frame}()},
\code{\link{is_parseable.default}()},
\code{\link{promote.default}()},
\code{\link{read_yamlet}()},
\code{\link{resolve.decorated}()},
\code{\link{selected.default}()},
\code{\link{write_yamlet}()}
}
\concept{interface}
\concept{modify}
